@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2008 Jean-Franois Moine
+ * Copyright (C) 1998-2009 Jean-Franois Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -69,14 +69,12 @@ static struct abcsym *process_pscomment(struct abcsym *as);
 static void set_tuplet(struct SYMBOL *s);
 static void sym_link(struct SYMBOL *s, int type);
 
-/* -- get the weight of a symbol -- */
-static int get_weight(struct SYMBOL *s)
-{
+/* -- weight of the symbols -- */
 static signed char w_tb[15] = {	/* !! index = symbol type !! */
 	0,
 	9,	/* 1- note / rest */
 	1,	/* 2- space */
-	3,	/* 3- bar (3 or -1) */
+	3,	/* 3- bar 3 !! used in sort_all !! */
 	2,	/* 4- clef */
 	5,	/* 5- timesig */
 	4,	/* 6- keysig */
@@ -90,13 +88,6 @@ static signed char w_tb[15] = {	/* !! index = symbol type !! */
 	6	/* 14- stbrk */
 };
 
-#if 0
-	if (s->type == BAR && s->prev != 0 && s->prev->type == BAR)
-		return -1;	/* don't separate the bars */
-#endif
-	return w_tb[s->type];
-}
-
 /* -- expand a multi-rest into single rests and measure bars -- */
 static void mrest_expand(struct SYMBOL *s)
 {
@@ -120,7 +111,7 @@ static void mrest_expand(struct SYMBOL *s)
 	p_voice = &voice_tb[s->voice];
 	p_voice->last_sym = s;
 	p_voice->time = s->time + dt;
-	s2 = 0;				/* (compiler warning) */
+	s2 = s;
 	while (--nb > 0) {
 		s2 = sym_add(p_voice, BAR);
 		s2->as.type = ABC_T_BAR;
@@ -143,83 +134,118 @@ static void mrest_expand(struct SYMBOL *s)
 /* -- sort all symbols by time and vertical sequence -- */
 static void sort_all(void)
 {
+	struct SYSTEM *sy;
 	struct SYMBOL *s, *prev;
 	struct VOICE_S *p_voice;
 	int fl, voice, time, w, wmin, multi, mrest_time;
-	int nv, nb;
+	int nv, nb, r, sysadv;
 	struct SYMBOL *vtb[MAXVOICE];
-	signed char w_tb[MAXVOICE];
+	signed char vn[MAXVOICE];	/* voice indexed by range */
 
 /*	memset(vtb, 0, sizeof vtb); */
 	mrest_time = -1;
 	multi = first_voice->next != 0;
-	for (p_voice = first_voice; p_voice != 0; p_voice = p_voice->next) {
+	for (p_voice = first_voice; p_voice != 0; p_voice = p_voice->next)
 		vtb[p_voice - voice_tb] = s = p_voice->sym;
-		if (multi && s != 0 && s->type == MREST && mrest_time < s->time)
-			mrest_time = s->time;
-	}
+
+	/* initialize the voice order */
+	sy = cursys;
+	sysadv = 1;
 	prev = 0;
-	fl = 1;			/* set start of sequence */
+	fl = 1;				/* set start of sequence */
 	for (;;) {
+		if (sysadv) {
+/*fixme: should set multi here*/
+			sysadv = 0;
+			memset(vn, -1, sizeof vn);
+			for (voice = 0; voice < MAXVOICE; voice++) {
+				r = sy->voice[voice].range;
+				if (r < 0)
+					continue;
+				vn[r] = voice;
+			}
+		}
 
 		/* search the min time and symbol weight */
 		wmin = time = (unsigned) ~0 >> 1;	/* max int */
 		nv = nb = 0;
-		for (p_voice = first_voice; p_voice != 0; p_voice = p_voice->next) {
-			voice = p_voice - voice_tb;
+		for (r = 0; r < MAXVOICE; r++) {
+			voice = vn[r];
+			if (voice < 0)
+				break;
 			s = vtb[voice];
 			if (s == 0 || s->time > time)
 				continue;
-			w = get_weight(s);
-			w_tb[voice] = w;
+			w = w_tb[s->type];
 			if (s->time < time) {
 				time = s->time;
 				wmin = w;
-			} else if (w < wmin)
+				nb = 0;
+			} else if (w < wmin) {
 				wmin = w;
+				nb = 0;
+			}
 			if (!(s->sflags & S_SECOND)) {
 				nv++;
 				if (s->type == BAR)
 					nb++;
 			}
+			if (multi && s->type == MREST)
+				mrest_time = time;
 		}
 		if (wmin > 127)
 			break;			/* done */
 
+#if 0
 		/* align the measure bars */
 		if (nb != 0 && nb != nv) {	/* if other symbol than bars */
-			wmin = time = (unsigned) ~0 >> 1;
-			for (p_voice = first_voice; p_voice != 0; p_voice = p_voice->next) {
-				voice = p_voice - voice_tb;
+			wmin = (unsigned) ~0 >> 1;
+			for (r = 0; r < MAXVOICE; r++) {
+				voice = vn[r];
+				if (voice < 0)
+					break;
 				s = vtb[voice];
 				if (s == 0 || s->time > time
 				    || s->type == BAR)
 					continue;
-				w = w_tb[voice];
-				if (s->time < time) {
-					time = s->time;
-					wmin = w;
-				} else if (w < wmin)
+				w = w_tb[s->type];
+				if (w < wmin)
 					wmin = w;
 			}
+			if (wmin > 127)
+				wmin = w_tb[BAR];
 		}
+#endif
 
 		/* if some multi-rest and many voices, expand */
 		if (time == mrest_time) {
-			for (p_voice = first_voice; p_voice != 0; p_voice = p_voice->next) {
-				voice = p_voice - voice_tb;
+			nb = 0;
+			for (r = 0; r < MAXVOICE; r++) {
+				voice = vn[r];
+				if (voice < 0)
+					break;
 				s = vtb[voice];
-				w = w_tb[voice];
-				if (s == 0 || s->time != time || w != wmin)
+				if (s == 0 || s->time != time)
+					continue;
+				w = w_tb[s->type];
+				if (w != wmin)
 					continue;
 				if (s->type != MREST) {
-					mrest_time = 0;	/* some note or rest */
+					mrest_time = -1;	/* some note or rest */
+					break;
+				}
+				if (nb == 0)
+					nb = s->as.u.bar.len;
+				else if (nb != s->as.u.bar.len) {
+					mrest_time = -1;	/* different duration */
 					break;
 				}
 			}
-			if (!mrest_time) {
-				for (p_voice = first_voice; p_voice != 0; p_voice = p_voice->next) {
-					voice = p_voice - voice_tb;
+			if (mrest_time < 0) {
+				for (r = 0; r < MAXVOICE; r++) {
+					voice = vn[r];
+					if (voice < 0)
+						break;
 					s = vtb[voice];
 					if (s != 0 && s->type == MREST)
 						mrest_expand(s);
@@ -228,11 +254,15 @@ static void sort_all(void)
 		}
 
 		/* link the vertical sequence */
-		for (p_voice = first_voice; p_voice != 0; p_voice = p_voice->next) {
-			voice = p_voice - voice_tb;
+		for (r = 0; r < MAXVOICE; r++) {
+			voice = vn[r];
+			if (voice < 0)
+				break;
 			s = vtb[voice];
-			w = w_tb[voice];
-			if (s == 0 || s->time != time || w != wmin)
+			if (s == 0 || s->time != time)
+				continue;
+			w = w_tb[s->type];
+			if (w != wmin)
 				continue;
 			if (fl) {
 				fl = 0;
@@ -244,13 +274,14 @@ static void sort_all(void)
 				    && (s->sflags & S_SECOND)
 				    && prev->type != BAR
 				    && !(s->as.flags & ABC_F_INVIS))
-					error(1, s,
-					      "Bad measure bar");
+					error(1, s, "Bad measure bar");
 			} else	tsfirst = s;
 			prev = s;
 			vtb[voice] = s->next;
-			if (multi && s->next != 0 && s->next->type == MREST)
-				mrest_time = s->next->time;
+			if (s->type == STAVES) {
+				sy = sy->next;
+				sysadv = 1;
+			}
 		}
 		fl = wmin;	/* start a new sequence if some space */
 	}
@@ -258,12 +289,14 @@ static void sort_all(void)
 	/* if no bar or format_change at end of tune, add a dummy symbol */
 	if (prev != 0 && prev->type != BAR && prev->type != FMTCHG) {
 		s = info['T' - 'A'];
-		prev->ts_next = s;
-		s->ts_prev = prev;
 		s->type = FMTCHG;
 		s->u = -1;
 		s->sflags = S_SEQST;
 		s->time = prev->time + prev->dur;
+		s->next = 0;
+		s->ts_next = 0;
+		prev->ts_next = s;
+		s->ts_prev = prev;
 		for (;;) {
 			prev->sflags &= ~S_EOLN;
 			if (prev->sflags & S_SEQST)
@@ -281,6 +314,8 @@ static void voice_compress(void)
 	int sflags;
 
 	for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
+		if (p_voice->ignore)
+			continue;
 		for (s = p_voice->sym; s != 0; s = s->next) {
 			if (s->time >= staves_found)
 				break;
@@ -289,15 +324,50 @@ static void voice_compress(void)
 		sflags = 0;
 		for ( ; s != 0; s = s->next) {
 			switch (s->type) {
+			case FMTCHG:
+				s2 = s->extra;
+				if (s2 != 0) {	/* dummy format */
+					if (ns == 0)
+						ns = s2;
+					if (s->prev != 0) {
+						s->prev->next = s2;
+						s2->prev = s->prev;
+					}
+					while (s2->next != 0)
+						s2 = s2->next;
+					if (s->next == 0) {
+						ns = 0;
+						break;
+					}
+					s->next->prev = s2;
+					s2->next = s->next;
+				}
+				/* fall thru */
 			case TEMPO:
 			case PART:
 			case TUPLET:
-			case FMTCHG:
-/*fixme:and STAVES?*/
 				if (ns == 0)
 					ns = s;
 				sflags |= s->sflags;
 				continue;
+			case MREST:		/* don't shift P: and Q: */
+				if (ns == 0)
+					continue;
+				s2 = (struct SYMBOL *) getarena(sizeof *s);
+				memset(s2, 0, sizeof *s2);
+				s2->type = SPACE;
+				s2->as.u.note.lens[1] = -1;
+				s2->as.flags = ABC_F_INVIS;
+				s2->voice = s->voice;
+				s2->staff = s->staff;
+				s2->time = s->time;
+				s2->sflags = s->sflags;
+				s2->next = s;
+				s2->prev = s->prev;
+				s2->prev->next = s2;
+				s->prev = s2;
+				s = s2;
+				break;
 			}
 			if (s->as.flags & ABC_F_GRACE) {
 				if (ns == 0)
@@ -328,11 +398,14 @@ static void voice_compress(void)
 			ns = 0;
 			sflags = 0;
 		}
+
+		/* when symbols with no space at end of tune,
+		 * add a dummy format */
 		if (ns != 0) {
 			s = sym_add(p_voice, FMTCHG);
 			s->u = -1;		/* nothing */
 			s->extra = ns;
-			s->prev->next = 0;
+			s->prev->next = 0;	/* unlink */
 			if ((s->prev = ns->prev) != 0)
 				s->prev->next = s;
 			else	p_voice->sym = s;
@@ -524,7 +597,7 @@ static void staves_init(void)
 }
 
 /* -- initialize the voices and staves -- */
-/* this routine is called starting the generation */
+/* this routine is called when starting the generation */
 static void system_init(void)
 {
 	voice_compress();
@@ -638,15 +711,12 @@ static void gchord_adjust(struct SYMBOL *s)
 		case '/':
 			q = p + 1;
 			break;
-		case ';':
-			if (p[-1] == '\\' || p[1] == '\0')
-				break;
-			/* fall thru */
 		case '\n':
-			if (p[1] != '\0' && strchr("^_<>@", p[1]) != 0) {
+			if (strchr("^_<>@", p[1]) != 0) {
 				annot = 1;
 				p++;
 			} else	annot = cfmt.freegchord;
+			q = p + 1;
 			break;
 		}
 		p++;
@@ -761,7 +831,9 @@ static char *get_lyric(char *p)
 		}
 
 		/* store word in next note */
-		while (s != 0 && s->as.type != ABC_T_NOTE)
+		while (s != 0
+		       && (s->as.type != ABC_T_NOTE
+			   || (s->as.flags & ABC_F_GRACE)))
 			s = s->next;
 		if (s == 0)
 			return "Too many words in lyric line";
@@ -798,10 +870,13 @@ static char *get_lyric(char *p)
 		}
 		s = s->next;
 	}
-	while (s != 0 && s->as.type != ABC_T_NOTE)
+	while (s != 0
+	       && (s->as.type != ABC_T_NOTE
+		   || (s->as.flags & ABC_F_GRACE)))
 		s = s->next;
 	if (s != 0)
 		return "Not enough words for lyric line";
+	curvoice->have_ly = 1;
 	return 0;
 }
 
@@ -830,6 +905,8 @@ static char tx_wrong_dur[] = "Wrong duration in voice overlay";
 
 	/* treat the end of overlay */
 	p_voice = curvoice;
+	if (p_voice->ignore)
+		return;
 	if (s->as.type == ABC_T_BAR
 	    || s->as.u.v_over.type == V_OVER_E)  {
 		over_bar = 0;
@@ -951,14 +1028,14 @@ static void get_staves(struct SYMBOL *s)
 		p_voice->ignore = 0;
 		p_voice->time = maxtime;
 	}
-	if (flags == 0				/* if first %%staves */
+	if (flags == 0			/* if first %%staves */
 	    || (maxtime == 0 && staves_found < 0)) {
 		for (voice = 0; voice < MAXVOICE; voice++)
 			parsys->voice[voice].range = -1;
-	} else {				/* else, have a new staff system */
+	} else {			/* else, have a new staff system */
 		parsys->nstaff = nstaff;
 		sym_link(s, STAVES);
-		s->as.state = ABC_S_HEAD;	/* (output PS sequences immediately) */
+		s->as.state = ABC_S_HEAD; /* (output PS sequences immediately) */
 		system_new();
 	}
 	staves_found = maxtime;
@@ -966,7 +1043,9 @@ static void get_staves(struct SYMBOL *s)
 	/* initialize the voices */
 	dup_voice = MAXVOICE;
 	range = 0;
-	for (i = 0, p_staff = s->as.u.staves;
+	p_staff = s->as.u.staves;
+	parsys->top_voice = p_staff->voice;
+	for (i = 0;
 	     i < MAXVOICE && p_staff->voice >= 0;
 	     i++, p_staff++) {
 		voice = p_staff->voice;
@@ -1035,7 +1114,6 @@ static void get_staves(struct SYMBOL *s)
 		if (flags & FL_VOICE) {
 			p_voice->floating = 1;
 			p_voice->second = 1;
-			parsys->voice[voice].second = 1;
 		} else {
 #if MAXSTAFF < MAXVOICE
 			if (staff >= MAXSTAFF - 1) {
@@ -1055,7 +1133,6 @@ static void get_staves(struct SYMBOL *s)
 				voice = p_staff->voice;
 				p_voice = &voice_tb[voice];
 				p_voice->second = 1;
-				parsys->voice[voice].second = 1;
 				p_voice->staff = p_voice->cstaff
 					= parsys->voice[voice].staff
 					= staff;
@@ -1068,6 +1145,8 @@ static void get_staves(struct SYMBOL *s)
 	if (staff < 0)
 		staff = 0;
 	nstaff = staff;
+	for (voice = 0; voice < MAXVOICE; voice++)
+		parsys->voice[voice].second = voice_tb[voice].second;
 }
 
 /* -- re-initialize all potential voices -- */
@@ -1134,7 +1213,9 @@ static void get_info(struct SYMBOL *s,
 		over_time = -1;
 		over_bar = 0;
 		reset_gen();
-		curvoice = first_voice;		/* switch to the 1st voice */
+
+		/* switch to the 1st voice */
+		curvoice = &voice_tb[parsys->top_voice];
 		break;
 	case 'L':
 		break;
@@ -1153,7 +1234,7 @@ static void get_info(struct SYMBOL *s,
 			struct VOICE_S *p_voice;
 
 			p_voice = curvoice;
-			curvoice = first_voice;
+			curvoice = &voice_tb[parsys->top_voice];
 			sym_link(s, PART);
 			if (p_voice->time != curvoice->time) {
 				error(1, s, "Misplaced P:");
@@ -1170,8 +1251,8 @@ static void get_info(struct SYMBOL *s,
 		}
 		break;
 	case 'Q':
-		if (curvoice != first_voice)	/* tempo only for first voice */
-			break;
+		if (curvoice != &voice_tb[parsys->top_voice])
+			break;		/* tempo only for first voice */
 		switch (s->as.state) {
 		case ABC_S_GLOBAL:
 		case ABC_S_HEAD:
@@ -1197,7 +1278,7 @@ static void get_info(struct SYMBOL *s,
 			bskip(cfmt.musicspace + 0.2 CM);
 			voice_init();
 			reset_gen();		/* (display the time signature) */
-			curvoice = first_voice;
+			curvoice = &voice_tb[parsys->top_voice];
 			break;
 		}
 		break;
@@ -1224,7 +1305,6 @@ static void get_info(struct SYMBOL *s,
 		       p++;
 		if ((p = get_lyric(p)) != 0)
 			error(1, s, "%s", p);
-		curvoice->have_ly = 1;
 		break;
 	case 'W':
 		if (s->as.state == ABC_S_GLOBAL)
@@ -1240,6 +1320,7 @@ newtune:
 		if (info_type == 'T')
 			info['T' - 'A'] = s;
 		memcpy(&deco_tune, &deco_glob, sizeof deco_tune);
+		lvlarena(1);
 		break;
 	default:
 addinfo:
@@ -1337,7 +1418,7 @@ static void get_bar(struct SYMBOL *s)
 
 		/* remove the invisible repeat bars when no shift is needed */
 		if (bar_type == B_OBRA
-		    && (curvoice == first_voice
+		    && (curvoice == &voice_tb[parsys->top_voice]
 			|| (parsys->staff[curvoice->staff - 1].flags & STOP_BAR)
 			|| (s->sflags & S_NOREPBRA))) {
 			if (s2->type == BAR && s2->as.text == 0) {
@@ -1378,7 +1459,7 @@ static void get_bar(struct SYMBOL *s)
 	/* set some flags */
 	switch (bar_type) {
 	case B_OBRA:
-	case B_CBRA:
+/*	case B_CBRA:			thick bar or end of repeat braket */
 	case (B_OBRA << 4) + B_CBRA:
 		s->as.flags |= ABC_F_INVIS;
 		break;
@@ -1391,7 +1472,7 @@ static void get_bar(struct SYMBOL *s)
 			s->sflags |= S_RRBAR;
 	}
 	if (bar_number != 0
-	    && curvoice == first_voice) {
+	    && curvoice == &voice_tb[parsys->top_voice]) {
 		s->u = bar_number;		/* set the new bar number */
 		bar_number = 0;
 	}
@@ -1437,6 +1518,7 @@ void do_tune(struct abctune *t,
 	int i;
 
 	/* initialize */
+	lvlarena(0);
 	nstaff = 0;
 	staves_found = -1;
 	memset(staff_tb, 0, sizeof staff_tb);
@@ -1457,10 +1539,12 @@ void do_tune(struct abctune *t,
 	voice_tb[0].name = "1";		/* implicit voice */
 	set_tblt(first_voice);
 	micro_tb = t->micro_tb;		/* microtone values */
+	abc2win = 0;
 
 	parsys = 0;
 	system_new();			/* create the 1st staff system */
-	parsys->voice[0].range = 0;	/* implicit voice */
+	parsys->top_voice =
+		parsys->voice[0].range = 0;	/* implicit voice */
 
 	if (cfmt.oneperpage) {
 		use_buffer = 0;
@@ -1546,12 +1630,29 @@ void do_tune(struct abctune *t,
 				curvoice->last_sym->sflags |= S_EOLN;
 			if (!cfmt.alignbars)
 				continue;
+			while (as->next != 0) {	/* treat the lyrics */
+				if (as->next->type != ABC_T_INFO)
+					break;
+				switch (as->next->text[0]) {
+				case 'w':
+					get_info((struct SYMBOL *) as->next,
+						 'w', t);
+					/* fall thru */
+				case 'd':
+				case 's':
+					as = as->next;
+					s = (struct SYMBOL *) as;
+					continue;
+				}
+				break;
+			}
 			i = (curvoice - voice_tb) + 1;
 			if (i < cfmt.alignbars) {
 				curvoice = &voice_tb[i];
 				continue;
 			}
-			gen_ly(1);
+			generate();
+			buffer_eob();
 			curvoice = &voice_tb[0];
 			continue;
 		case ABC_T_MREST: {
@@ -1562,8 +1663,9 @@ void do_tune(struct abctune *t,
 				curvoice->time += dur;
 				break;
 			}
-			s->dur = dur;
 			sym_link(s, MREST);
+			s->dur = dur;
+			curvoice->time += dur;
 			if (s->as.text != 0)		/* adjust the */
 				gchord_adjust(s);	/* guitar chords */
 			if (s->as.u.bar.dc.n > 0)
@@ -1580,6 +1682,7 @@ void do_tune(struct abctune *t,
 			}
 			if (curvoice->ignore)
 				break;
+			n = as->u.bar.len;
 			if (curvoice->second) {
 				curvoice->time += curvoice->wmeasure * n;
 				break;
@@ -1591,7 +1694,6 @@ void do_tune(struct abctune *t,
 			s2->as.flags |= ABC_F_INVIS;
 			s2->dur = curvoice->wmeasure;
 			curvoice->time += s2->dur;
-			n = as->u.bar.len;
 			if (n == 1) {
 				as->next->u.bar.len = n; /* <n> in the next bar */
 				break;
@@ -1644,6 +1746,8 @@ void do_tune(struct abctune *t,
 		cfmt = dfmt;	/* restore the format at start of tune */
 		info['X' - 'A'] = 0;
 	}
+	clrarena(1);
+	
 }
 
 /* -- get a clef definition (in K: or V:) -- */
@@ -1739,6 +1843,45 @@ static void get_clef(struct SYMBOL *s)
 		p_voice->forced_clef = 1;		/* don't change */
 }
 
+/* -- set the accidentals when K: with modified accidentals -- */
+static void set_acc(struct SYMBOL *s)
+{
+	int i, j, nacc;
+	char accs[8], pits[8];
+	static char sharp_tb[8] = {26, 23, 27, 24, 21, 25, 22};
+	static char flat_tb[8] = {22, 25, 21, 24, 20, 23, 26};
+
+	if (s->as.u.key.sf > 0) {
+		for (nacc = 0; nacc < s->as.u.key.sf; nacc++) {
+			accs[nacc] = A_SH;
+			pits[nacc] = sharp_tb[nacc];
+		}
+	} else {
+		for (nacc = 0; nacc < -s->as.u.key.sf; nacc++) {
+			accs[nacc] = A_FT;
+			pits[nacc] = flat_tb[nacc];
+		}
+	}
+	for (i = 0; i < s->as.u.key.nacc; i++) {
+		for (j = 0; j < nacc; j++) {
+			if ((pits[j] - s->as.u.key.pits[i]) % 7 == 0) {
+				accs[j] = s->as.u.key.accs[i];
+				break;
+			}
+		}
+		if (j == nacc) {
+			accs[j] = s->as.u.key.accs[i];
+			pits[j] = s->as.u.key.pits[i];
+			nacc++;		/* cannot overflow */
+		}
+	}
+	for (i = 0; i < nacc; i++) {
+		s->as.u.key.accs[i] = accs[i];
+		s->as.u.key.pits[i] = pits[i];
+	}
+	s->as.u.key.nacc = nacc;
+}
+
 /* -- get a key signature definition (K:) -- */
 static void get_key(struct SYMBOL *s)
 {
@@ -1748,6 +1891,10 @@ static void get_key(struct SYMBOL *s)
 
 	if (s->as.u.key.empty == 1)
 		return;				/* clef only */
+	if (s->as.u.key.sf != 0
+	    && !s->as.u.key.exp
+	    && s->as.u.key.nacc != 0)
+		set_acc(s);
 	switch (s->as.state) {
 	case ABC_S_HEAD:
 		for (i = MAXVOICE, p_voice = voice_tb;
@@ -1762,10 +1909,11 @@ static void get_key(struct SYMBOL *s)
 		break;
 	case ABC_S_TUNE:
 	case ABC_S_EMBED:
-		if (curvoice->last_sym == 0) {
+		if (curvoice->last_sym == 0
+		    && curvoice->time == 0) {
 
 			/* if first symbol of the first voice, apply to all voices */
-			if (curvoice == first_voice
+			if (curvoice == &voice_tb[parsys->top_voice]
 			    && s->as.state == ABC_S_TUNE) {	/* (not embedded) */
 				for (i = MAXVOICE, p_voice = voice_tb;
 				     --i >= 0;
@@ -1795,6 +1943,8 @@ static void get_key(struct SYMBOL *s)
 		s2 = curvoice->last_sym;
 		if (s2 != 0 && s2->type == TIMESIG) {
 			curvoice->last_sym = s2->prev;
+			if (curvoice->last_sym == 0)
+				curvoice->sym = 0;
 			sym_link(s, KEYSIG);
 			s->next = s2;
 			s2->prev = s;
@@ -1828,10 +1978,11 @@ static void get_meter(struct SYMBOL *s)
 	case ABC_S_TUNE:
 	case ABC_S_EMBED:
 		curvoice->wmeasure = s->as.u.meter.wmeasure;
-		if (curvoice->last_sym == 0) {
+		if (curvoice->last_sym == 0
+		    && curvoice->time == 0) {
 
 			/* if first symbol of the first voice, apply to all voices */
-			if (curvoice == first_voice
+			if (curvoice == &voice_tb[parsys->top_voice]
 			    && s->as.state == ABC_S_TUNE) {	/* (not embedded) */
 				for (i = MAXVOICE, p_voice = voice_tb;
 				     --i >= 0;
@@ -1917,6 +2068,9 @@ static void get_voice(struct SYMBOL *s)
 	if (s->as.u.voice.lyrics != 0)
 		p_voice->ly_pos = s->as.u.voice.lyrics != 2
 			? s->as.u.voice.lyrics : 0;
+	if (s->as.u.voice.gchord != 0)
+		p_voice->gchord = s->as.u.voice.gchord != 2
+			? s->as.u.voice.gchord : 0;
 	if (s->as.u.voice.scale != 0)
 		p_voice->scale = s->as.u.voice.scale;
 
@@ -1944,22 +2098,22 @@ static void get_note(struct SYMBOL *s)
 	else {			/* grace note - adjust its duration */
 		int div;
 
-		s->dur = 0;
 		s->stem = curvoice->gstem;
 		if (!curvoice->key.bagpipe) {
 			div = 4;
-			if (s->prev == 0) {
-				if (s->as.next == 0
-				    || s->as.next->type != ABC_T_NOTE
-				    || !(s->as.next->flags & ABC_F_GRACE))
-					div = 2;
+			if (curvoice->last_sym == 0
+			    || !(curvoice->last_sym->as.flags & ABC_F_GRACE)) {
+				if (s->as.flags & ABC_F_GR_END)
+					div = 2;	/* one grace note */
 			}
 		} else	div = 8;
 		for (i = 0; i <= m; i++)
 			s->as.u.note.lens[i] /= div;
+		s->dur /= div;
 	}
-	sym_link(s,
-		 s->as.u.note.lens[0] != 0 ? NOTEREST : SPACE);
+	sym_link(s,  s->as.u.note.lens[0] != 0 ? NOTEREST : SPACE);
+	if (!(s->as.flags & ABC_F_GRACE))
+		curvoice->time += s->dur;
 	s->nohdix = -1;
 
 	/* convert the decorations */
@@ -2013,34 +2167,40 @@ static void get_note(struct SYMBOL *s)
 			identify_note(s, l, &head, &dots, &nflags);
 			s->head = head;
 			s->dots = dots;
-			if (!(s->sflags & S_TREM)) {
-				s->nflags = nflags;
-				for (i = 1; i <= m; i++) {
-					if (s->as.u.note.lens[i] == l)
-						continue;
-					identify_note(s, s->as.u.note.lens[i],
-						      &head, &dots, &nflags);
-					if (head > s->head)
-						s->head = head;
-					if (dots > s->dots)
-						s->dots = dots;
-					if (nflags > s->nflags)
-						s->nflags = nflags;
-				}
-				if (s->sflags & S_XSTEM)
-					s->nflags = 0;	/* word start+end */
-			} else {	/* 2nd tremolo note */
-				prev->head = head;
-				if (head >= H_OVAL) {
-					s->as.flags |= ABC_F_STEMLESS;
-					prev->as.flags |= ABC_F_STEMLESS;
-				}
+			s->nflags = nflags;
+			for (i = 1; i <= m; i++) {
+				if (s->as.u.note.lens[i] == l)
+					continue;
+				identify_note(s, s->as.u.note.lens[i],
+					      &head, &dots, &nflags);
+				if (head > s->head)
+					s->head = head;
+				if (dots > s->dots)
+					s->dots = dots;
+				if (nflags > s->nflags)
+					s->nflags = nflags;
 			}
+			if (s->sflags & S_XSTEM)
+				s->nflags = 0;		/* word start+end */
 		}
 	}
 	if (s->nflags <= -2)
 		s->as.flags |= ABC_F_STEMLESS;
 
+	if (s->sflags & (S_TREM1 | S_TREM2)) {
+		if (s->nflags > 0)
+			s->nflags += s->u;
+		else
+			s->nflags = s->u;
+		if ((s->sflags & S_TREM2) && (s->sflags & S_BEAM_END)) {
+			prev->head = s->head;
+			prev->u = s->u;
+			prev->nflags = s->nflags;
+			prev->as.flags |= (s->as.flags & ABC_F_STEMLESS);
+		}
+	}
+
+
 	for (i = 0; i <= m; i++) {
 		if (s->as.u.note.sl1[i] != 0)
 			s->sflags |= S_SL1;
@@ -2536,37 +2696,73 @@ irepeat:
 			cfmt.alignbars = nstaff + 1;
 		first_voice = curvoice = &voice_tb[0];
 		for (i = 0; i < cfmt.alignbars; i++) {
-			parsys->staff[i].flags |= STOP_BAR;
 			voice_tb[i].staff = voice_tb[i].cstaff = i;
+			voice_tb[i].next = &voice_tb[i + 1];
+			parsys->staff[i].flags |= STOP_BAR;
 			parsys->voice[i].staff = i;
 			parsys->voice[i].range = i;
 		}
+		voice_tb[i - 1].next = 0;
 		nstaff = i;
 	}
 	return as;
 }
 
 /* -- set the duration of notes/rests in a tuplet -- */
+/*fixme: KO if voice change*/
+/*fixme: one nesting level only*/
+/*fixme: KO if in a grace sequence*/
 static void set_tuplet(struct SYMBOL *t)
 {
-	struct abcsym *as, *first;
+	struct abcsym *as;
 	struct SYMBOL *s;
 	int l, r, lplet, grace;
 
 	r = t->as.u.tuplet.r_plet;
 	grace = t->as.flags & ABC_F_GRACE;
+
 	l = 0;
-	first = 0;
 	for (as = t->as.next; as != 0; as = as->next) {
-/*fixme: KO if voice change..*/
+		if (as->type == ABC_T_TUPLET) {
+			struct abcsym *as2;
+			int l2, r2;
+
+			r2 = as->u.tuplet.r_plet;
+			l2 = 0;
+			for (as2 = as->next; as2 != 0; as2 = as2->next) {
+				if (as2->type != ABC_T_NOTE
+				    && as2->type != ABC_T_REST)
+					continue;
+				if (as2->u.note.lens[0] == 0)
+					continue;
+				if (grace ^ (as2->flags & ABC_F_GRACE))
+					continue;
+				s = (struct SYMBOL *) as2;
+				l2 += s->dur;
+				if (--r2 <= 0)
+					break;
+			}
+			l2 = l2 * as->u.tuplet.q_plet
+					/ as->u.tuplet.p_plet;
+			((struct SYMBOL *) as)->u = l2;
+			l += l2;
+			r -= as->u.tuplet.r_plet;
+			if (r == 0)
+				break;
+			if (r < 0) {
+				error(1, t,
+				      "Bad nested tuplet");
+				break;
+			}
+			as = as2;
+			continue;
+		}
 		if (as->type != ABC_T_NOTE && as->type != ABC_T_REST)
 			continue;
 		if (as->u.note.lens[0] == 0)	/* space ('y') */
 			continue;
 		if (grace ^ (as->flags & ABC_F_GRACE))
 			continue;
-		if (first == 0)
-			first = as;
 		s = (struct SYMBOL *) as;
 		l += s->dur;
 		if (--r <= 0)
@@ -2577,12 +2773,39 @@ static void set_tuplet(struct SYMBOL *t)
 		      "End of tune found inside a tuplet");
 		return;
 	}
-	lplet = (l * t->as.u.tuplet.q_plet) / t->as.u.tuplet.p_plet;
+	if (t->u != 0)		/* if nested tuplet */
+		lplet = t->u;
+	else	lplet = (l * t->as.u.tuplet.q_plet) / t->as.u.tuplet.p_plet;
 	r = t->as.u.tuplet.r_plet;
-	for (as = first; as != 0; as = as->next) {
+	for (as = t->as.next; as != 0; as = as->next) {
 		int olddur;
 
-/*fixme: KO if voice change..*/
+		if (as->type == ABC_T_TUPLET) {
+			int r2;
+
+			r2 = as->u.tuplet.r_plet;
+			s = (struct SYMBOL *) as;
+			olddur = s->u;
+			s->u = (olddur * lplet) / l;
+			l -= olddur;
+			lplet -= s->u;
+			r -= r2;
+			for (;;) {
+				as = as->next;
+				if (as->type != ABC_T_NOTE
+				    && as->type != ABC_T_REST)
+					continue;
+				if (as->u.note.lens[0] == 0)
+					continue;
+				if (grace ^ (as->flags & ABC_F_GRACE))
+					continue;
+				if (--r2 <= 0)
+					break;
+			}
+			if (r <= 0)
+				goto done;
+			continue;
+		}
 		if (as->type != ABC_T_NOTE && as->type != ABC_T_REST)
 			continue;
 		if (as->u.note.lens[0] == 0)
@@ -2593,13 +2816,12 @@ static void set_tuplet(struct SYMBOL *t)
 		s->sflags |= S_IN_TUPLET;
 		olddur = s->dur;
 		s->dur = (olddur * lplet) / l;
-		if (--r <= 0) {
+		if (--r <= 0)
 			break;
-		}
 		l -= olddur;
 		lplet -= s->dur;
 	}
-/*fixme: KO if in a grace sequence*/
+done:
 	if (grace)
 		error(1, t,
 		      "Tuplets in grace note sequence not yet treated");
@@ -2646,6 +2868,4 @@ static void sym_link(struct SYMBOL *s, int type)
 	s->voice = p_voice - voice_tb;
 	s->staff = p_voice->cstaff;
 	s->time = p_voice->time;
-	if (s->dur != 0)
-		p_voice->time += s->dur;
 }
